using System;
using System.Threading.Tasks;
using Microsoft.AspNetCore.SignalR;

namespace Duotris
{

    // Remember that a SignalR Hub is created for each received message!
    public class DuotrisHub : Hub
    {
        // The list of currently running games.
        private readonly IGameList matches;

        // The list of waiting players.
        private readonly IPlayerQueue playerQueue;

        // https://github.com/dotnet/aspnetcore/issues/48690
        private new IHubClients Clients;

        public DuotrisHub(IHubContext<DuotrisHub> context, IGameList matches, IPlayerQueue playerQueue)
        {
            this.Clients = context.Clients;
            this.matches = matches;
            this.playerQueue = playerQueue;
        }

        public override async Task OnConnectedAsync()
        {
            await Clients.Client(Protagonist).SendAsync("Message", "Waiting...");

            QueueForMatch(Context.ConnectionId);
            TryMakeMatch();

            await base.OnConnectedAsync();
        }

        public override async Task OnDisconnectedAsync(Exception? ex)
        {
            await Clients.Client(Antagonist).SendAsync("Message", "Your opponent disconnected");
            await base.OnDisconnectedAsync(ex);
        }

        private void QueueForMatch(string player)
        {
            lock (playerQueue)
            {
                Console.WriteLine($"Player {player} is waiting for a game.");
                playerQueue.Enqueue(player);
            }
        }

        private void TryMakeMatch()
        {
            lock (playerQueue)
            {
                if (playerQueue.Count >= 2)
                {
                    string player1 = playerQueue.Dequeue();
                    string player2 = playerQueue.Dequeue();

                    Console.WriteLine($"Starting a match for player {player1} and {player2}.");

                    // add players to a game and notify them
                    matches.Add(player1, player2);

                    Clients.Client(player1).SendAsync("Start");
                    Clients.Client(player2).SendAsync("Start");

                    Clients.Client(player1).SendAsync("Message", "Game started!");
                    Clients.Client(player2).SendAsync("Message", "Game started!");
                }
            }
        }

        public void Move(int[][] bucket, int score, int lines)
        {
            Clients.Client(Antagonist).SendAsync("Move", bucket, score, lines);
        }

        public void LinesCleared(int lines)
        {
            Clients.Client(Antagonist).SendAsync("LinesCleared", lines);
        }

        public void Gameover()
        {
            Clients.Client(Antagonist).SendAsync("GameOver");
        }

        private string Protagonist
        {
            get { return Context.ConnectionId; }
        }

        private string Antagonist
        {
            get { return matches[Protagonist]; }
        }
    }
}
